@@ -38,10 +38,6 @@
 #define MAX_FIFO_DATA_SIZE	(4096)
 #define DEFAULT_FIFO_DATA_SIZE	(MIN_FIFO_DATA_SIZE)
 
-#if defined(HAVE_SYS_SELECT_H)
-static const uint64_t wrap_mask = 0xffff000000000000ULL;
-#endif
-
 static const stress_help_t help[] = {
 	{ NULL,	"fifo N",		"start N workers exercising fifo I/O" },
 	{ NULL,	"fifo-data-size N",	"set fifo read/write size in bytes (default 8)" },
@@ -51,12 +47,16 @@ static const stress_help_t help[] = {
 };
 
 static const stress_opt_t opts[] = {
-	{ OPT_fifo_data_size, "fifo-data-size", TYPE_ID_SIZE_T_BYTES_FS, MIN_FIFO_DATA_SIZE, MAX_FIFO_DATA_SIZE, NULL },
+	{ OPT_fifo_data_size, "fifo-data-size", TYPE_ID_SIZE_T_BYTES_VM, MIN_FIFO_DATA_SIZE, MAX_FIFO_DATA_SIZE, NULL },
 	{ OPT_fifo_readers,   "fifo-readers",   TYPE_ID_UINT64, MIN_FIFO_READERS, MAX_FIFO_READERS, NULL },
 	END_OPT,
 };
 
-#if defined(HAVE_SYS_SELECT_H)
+#if defined(HAVE_SYS_SELECT_H) &&	\
+    defined(HAVE_MKFIFO)
+
+static const uint64_t wrap_mask = 0xffff000000000000ULL;
+
 /*
  *  fifo_spawn()
  *	spawn a process
@@ -207,7 +207,7 @@ redo_select:
 			/* Exercise mmap -> ENODEV */
 			ptr = mmap(NULL, args->page_size, PROT_READ,
 				MAP_PRIVATE, fd, 0);
-			if (ptr)
+			if (ptr != MAP_FAILED)
 				(void)munmap(ptr, args->page_size);
 		}
 		count++;
@@ -248,7 +248,8 @@ static int stress_fifo(stress_args_t *args)
 
 	s_pids = stress_sync_s_pids_mmap(MAX_FIFO_READERS);
 	if (s_pids == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, MAX_FIFO_READERS);
+		pr_inf_skip("%s: failed to mmap %d PIDs%s, skipping stressor\n",
+			args->name, MAX_FIFO_READERS, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -354,7 +355,7 @@ tidy_pids:
 
 const stressor_info_t stress_fifo_info = {
 	.stressor = stress_fifo,
-	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
+	.classifier = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -362,7 +363,7 @@ const stressor_info_t stress_fifo_info = {
 #else
 const stressor_info_t stress_fifo_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
+	.classifier = CLASS_PIPE_IO | CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
