@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-attribute.h"
+#include "core-builtin.h"
+#include "core-pragma.h"
+#include "core-target-clones.h"
 
 #if defined(HAVE_SYS_UIO_H)
 #include <sys/uio.h>
@@ -155,26 +159,6 @@ static const stress_hdd_opts_t hdd_opts[] = {
 	{ "utimes",	HDD_OPT_UTIMES, 0, 0, 0 },
 };
 
-static int stress_set_hdd_bytes(const char *opt)
-{
-	uint64_t hdd_bytes;
-
-	hdd_bytes = stress_get_uint64_byte_filesystem(opt, 1);
-	stress_check_range_bytes("hdd-bytes", hdd_bytes,
-		MIN_HDD_BYTES, MAX_HDD_BYTES);
-	return stress_set_setting("hdd-bytes", TYPE_ID_UINT64, &hdd_bytes);
-}
-
-static int stress_set_hdd_write_size(const char *opt)
-{
-	uint64_t hdd_write_size;
-
-	hdd_write_size = stress_get_uint64_byte(opt);
-	stress_check_range_bytes("hdd-write-size", hdd_write_size,
-		MIN_HDD_WRITE_SIZE, MAX_HDD_WRITE_SIZE);
-	return stress_set_setting("hdd-write-size", TYPE_ID_UINT64, &hdd_write_size);
-}
-
 #if defined(HAVE_FUTIMES)
 static void stress_hdd_utimes(const int fd)
 {
@@ -208,9 +192,12 @@ static ssize_t stress_hdd_write(
 	uint8_t *buf,
 	const off_t offset,
 	const uint64_t hdd_write_size,
-	const int hdd_flags)
+	const int hdd_flags,
+	double *hdd_write_bytes,
+	double *hdd_write_duration)
 {
-	ssize_t ret;
+	ssize_t ret = -1;
+	double t;
 
 #if defined(HAVE_FUTIMES)
 	if (hdd_flags & HDD_OPT_UTIMES)
@@ -225,6 +212,9 @@ static ssize_t stress_hdd_write(
 		size_t i;
 		uint8_t *data = buf;
 		const uint64_t sz = hdd_write_size / HDD_IO_VEC_MAX;
+#if defined(HAVE_PWRITEV2)
+		int pwitev2_flag = 0;
+#endif
 
 		for (i = 0; i < HDD_IO_VEC_MAX; i++) {
 			iov[i].iov_base = (void *)data;
@@ -236,34 +226,65 @@ static ssize_t stress_hdd_write(
 		switch (stress_mwc8modn(3)) {
 #if defined(HAVE_PWRITEV2)
 		case 0:
-			ret = pwritev2(fd, iov, HDD_IO_VEC_MAX, offset, 0);
+			t = stress_time_now();
+#if defined(RWF_HIPRI)
+			if (hdd_flags & HDD_OPT_O_DIRECT)
+				pwitev2_flag |= RWF_HIPRI;
+#endif
+			ret = pwritev2(fd, iov, HDD_IO_VEC_MAX, offset, pwitev2_flag);
+			if (ret > 0) {
+				(*hdd_write_duration) += stress_time_now() - t;
+				(*hdd_write_bytes) += (double)ret;
+			}
 			break;
 #endif
 #if defined(HAVE_PWRITEV)
 		case 1:
+			t = stress_time_now();
 			ret = pwritev(fd, iov, HDD_IO_VEC_MAX, offset);
+			if (ret > 0) {
+				(*hdd_write_duration) += stress_time_now() - t;
+				(*hdd_write_bytes) += (double)ret;
+			}
 			break;
 #endif
+#if defined(HAVE_WRITEV)
 		default:
+			t = stress_time_now();
 			if (lseek(fd, offset, SEEK_SET) < 0) {
 				ret = -1;
 			} else {
 				ret = writev(fd, iov, HDD_IO_VEC_MAX);
+				if (ret > 0) {
+					(*hdd_write_duration) += stress_time_now() - t;
+					(*hdd_write_bytes) += (double)ret;
+				}
 			}
 			break;
+#endif
 		}
 #else
+		t = stress_time_now();
 		if (lseek(fd, offset, SEEK_SET) < 0) {
 			ret = -1;
 		} else {
 			ret = write(fd, buf, (size_t)hdd_write_size);
+			if (ret > 0) {
+				(*hdd_write_duration) += stress_time_now() - t;
+				(*hdd_write_bytes) += (double)ret;
+			}
 		}
 #endif
 	} else {
+		t = stress_time_now();
 		if (lseek(fd, offset, SEEK_SET) < 0) {
 			ret = -1;
 		} else {
 			ret = write(fd, buf, (size_t)hdd_write_size);
+			if (ret > 0) {
+				(*hdd_write_duration) += stress_time_now() - t;
+				(*hdd_write_bytes) += (double)ret;
+			}
 		}
 	}
 
@@ -298,8 +319,13 @@ static ssize_t stress_hdd_read(
 	uint8_t *buf,
 	const off_t offset,
 	const uint64_t hdd_read_size,
-	const int hdd_flags)
+	const int hdd_flags,
+	double *hdd_read_bytes,
+	double *hdd_read_duration)
 {
+	ssize_t ret = -1;
+	double t;
+
 #if defined(HAVE_FUTIMES)
 	if (hdd_flags & HDD_OPT_UTIMES)
 		stress_hdd_utimes(fd);
@@ -324,27 +350,60 @@ static ssize_t stress_hdd_read(
 		switch (stress_mwc8modn(3)) {
 #if defined(HAVE_PREADV2)
 		case 0:
-			return preadv2(fd, iov, HDD_IO_VEC_MAX, offset, 0);
+			t = stress_time_now();
+			ret = preadv2(fd, iov, HDD_IO_VEC_MAX, offset, 0);
+			if (ret > 0) {
+				(*hdd_read_duration) += stress_time_now() - t;
+				(*hdd_read_bytes) += (double)ret;
+			}
+			return ret;
 #endif
 #if defined(HAVE_PREADV)
 		case 1:
-			return preadv(fd, iov, HDD_IO_VEC_MAX, offset);
+			t = stress_time_now();
+			ret = preadv(fd, iov, HDD_IO_VEC_MAX, offset);
+			if (ret > 0) {
+				(*hdd_read_duration) += stress_time_now() - t;
+				(*hdd_read_bytes) += (double)ret;
+			}
+			return ret;
 #endif
+#if defined(HAVE_READV)
 		default:
+			t = stress_time_now();
 			if (lseek(fd, offset, SEEK_SET) < 0)
 				return -1;
-			return readv(fd, iov, HDD_IO_VEC_MAX);
+			ret = readv(fd, iov, HDD_IO_VEC_MAX);
+			if (ret > 0) {
+				(*hdd_read_duration) += stress_time_now() - t;
+				(*hdd_read_bytes) += (double)ret;
+			}
+			return ret;
+#endif
 		}
 #else
+		t = stress_time_now();
 		if (lseek(fd, offset, SEEK_SET) < 0)
 			return -1;
-		return read(fd, buf, (size_t)hdd_read_size);
+		ret = read(fd, buf, (size_t)hdd_read_size);
+		if (ret > 0) {
+			(*hdd_read_duration) += stress_time_now() - t;
+			(*hdd_read_bytes) += (double)ret;
+		}
+		return ret;
 #endif
 	} else {
+		t = stress_time_now();
 		if (lseek(fd, offset, SEEK_SET) < 0)
 			return -1;
-		return read(fd, buf, (size_t)hdd_read_size);
+		ret = read(fd, buf, (size_t)hdd_read_size);
+		if (ret > 0) {
+			(*hdd_read_duration) += stress_time_now() - t;
+			(*hdd_read_bytes) += (double)ret;
+		}
+		return ret;
 	}
+	return ret;
 }
 
 /*
@@ -393,7 +452,8 @@ static void stress_hdd_invalid_read(const int fd, uint8_t *buf)
 	UNEXPECTED
 #endif
 
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    defined(HAVE_READV)
 	/* invalid readv fd */
 	VOID_RET(ssize_t, readv(-1, iov, HDD_IO_VEC_MAX));
 #endif
@@ -450,7 +510,8 @@ static void stress_hdd_invalid_write(const int fd, uint8_t *buf)
 	UNEXPECTED
 #endif
 
-#if defined(HAVE_SYS_UIO_H)
+#if defined(HAVE_SYS_UIO_H) &&	\
+    defined(HAVE_WRITEV)
 	/* invalid writev fd */
 	VOID_RET(ssize_t, writev(-1, iov, HDD_IO_VEC_MAX));
 #else
@@ -462,19 +523,26 @@ static void stress_hdd_invalid_write(const int fd, uint8_t *buf)
 }
 
 /*
- *  stress_set_hdd_opts
+ *  stress_hdd_opts
  *	parse --hdd-opts option(s) list
  */
-static int stress_set_hdd_opts(const char *opts)
+static void stress_hdd_opts(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
 {
 	char *str, *ptr, *token;
 	int hdd_flags = 0;
 	int hdd_oflags = 0;
 	bool opts_set = false;
 
-	str = stress_const_optdup(opts);
-	if (!str)
-		return -1;
+	(void)type_id;
+	(void)value;
+
+	str = stress_const_optdup(opt_arg);
+	if (!str) {
+		(void)fprintf(stderr, "%s option: cannot dup string '%s'\n",
+			opt_name, opt_arg);
+		longjmp(g_error_env, 1);
+		stress_no_return();
+	}
 
 	for (ptr = str; (token = strtok(ptr, ",")) != NULL; ptr = NULL) {
 		size_t i;
@@ -482,22 +550,25 @@ static int stress_set_hdd_opts(const char *opts)
 
 		for (i = 0; i < SIZEOF_ARRAY(hdd_opts); i++) {
 			if (!strcmp(token, hdd_opts[i].opt)) {
-				int exclude = hdd_flags & hdd_opts[i].exclude;
+				const int exclude = hdd_flags & hdd_opts[i].exclude;
+
 				if (exclude) {
 					int j;
 
 					for (j = 0; hdd_opts[j].opt; j++) {
 						if ((exclude & hdd_opts[j].flag) == exclude) {
 							(void)fprintf(stderr,
-								"hdd-opt option '%s' is not "
+								"%s option '%s' is not "
 								"compatible with option '%s'\n",
-								token,
+								opt_name, token,
 								hdd_opts[j].opt);
-							break;
+							free(str);
+							longjmp(g_error_env, 1);
+							stress_no_return();
 						}
 					}
 					free(str);
-					return -1;
+					return;
 				}
 				hdd_flags  |= hdd_opts[i].flag;
 				hdd_oflags |= hdd_opts[i].oflag;
@@ -506,29 +577,27 @@ static int stress_set_hdd_opts(const char *opts)
 			}
 		}
 		if (!opt_ok) {
-			(void)fprintf(stderr, "hdd-opt option '%s' not known, options are:", token);
+			(void)fprintf(stderr, "%s option '%s' not known, options are:", opt_name, token);
 			for (i = 0; i < SIZEOF_ARRAY(hdd_opts); i++)
-				(void)fprintf(stderr, "%s %s",
-					i == 0 ? "" : ",", hdd_opts[i].opt);
+				(void)fprintf(stderr, " %s", hdd_opts[i].opt);
 			(void)fprintf(stderr, "\n");
 			free(str);
-			return -1;
+			longjmp(g_error_env, 1);
+			stress_no_return();
 		}
 	}
 
-	stress_set_setting("hdd-flags", TYPE_ID_INT, &hdd_flags);
-	stress_set_setting("hdd-oflags", TYPE_ID_INT, &hdd_oflags);
-	stress_set_setting("hdd-opts-set", TYPE_ID_BOOL, &opts_set);
+	stress_set_setting("hdd", "hdd-flags", TYPE_ID_INT, &hdd_flags);
+	stress_set_setting("hdd", "hdd-oflags", TYPE_ID_INT, &hdd_oflags);
+	stress_set_setting("hdd", "hdd-opts-set", TYPE_ID_BOOL, &opts_set);
 	free(str);
-
-	return 0;
 }
 
 /*
  *  stress_hdd_advise()
  *	set posix_fadvise options
  */
-static int stress_hdd_advise(const stress_args_t *args, const int fd, const int flags)
+static int stress_hdd_advise(stress_args_t *args, const int fd, const int flags)
 {
 #if (defined(POSIX_FADV_SEQ) || defined(POSIX_FADV_RANDOM) ||		\
      defined(POSIX_FADV_NOREUSE) || defined(POSIX_FADV_WILLNEED) ||	\
@@ -560,18 +629,32 @@ static int stress_hdd_advise(const stress_args_t *args, const int fd, const int
  *  data_value()
  *	generate 8 bit data value for offsets and instance # into a test file
  */
-static inline uint8_t data_value(const uint64_t i, uint64_t j, const stress_args_t *args)
+static inline ALWAYS_INLINE uint8_t PURE OPTIMIZE3 data_value(
+	const uint64_t i,
+	uint64_t j,
+	const uint32_t instance)
 {
-	register uint8_t v = (uint8_t)(((i + j) >> 9) + i + j + args->instance);
+	return (uint8_t)(((i + j) >> 9) + i + j + instance);
+}
 
-	return v;
+static void OPTIMIZE3 TARGET_CLONES hdd_fill_buf(
+	uint8_t *buf,
+	const uint64_t buf_size,
+	const uint64_t i,
+	const uint32_t instance)
+{
+	register uint64_t j;
+
+	for (j = 0; j < buf_size; j++) {
+		buf[j] = data_value(i, j, instance);
+	}
 }
 
 /*
  *  stress_hdd
  *	stress I/O via writes
  */
-static int stress_hdd(const stress_args_t *args)
+static int stress_hdd(stress_args_t *args)
 {
 	uint8_t *buf = NULL;
 	void *alloc_buf;
@@ -579,12 +662,17 @@ static int stress_hdd(const stress_args_t *args)
 	int rc = EXIT_FAILURE;
 	ssize_t ret;
 	char filename[PATH_MAX];
-	size_t opt_index = 0;
-	uint64_t hdd_bytes = DEFAULT_HDD_BYTES;
+	size_t opt_index = 0, max_extents = 0;
+	uint64_t hdd_bytes, hdd_bytes_total = DEFAULT_HDD_BYTES;
 	uint64_t hdd_write_size = DEFAULT_HDD_WRITE_SIZE;
+	const uint32_t instance = args->instance;
 	int hdd_flags = 0, hdd_oflags = 0;
 	int flags, fadvise_flags;
 	bool opts_set = false;
+	double hdd_read_bytes = 0.0, hdd_read_duration = 0.0;
+	double hdd_write_bytes = 0.0, hdd_write_duration = 0.0;
+	double hdd_rdwr_bytes, hdd_rdwr_duration;
+	double rate;
 
 	(void)stress_get_setting("hdd-flags", &hdd_flags);
 	(void)stress_get_setting("hdd-oflags", &hdd_oflags);
@@ -593,16 +681,32 @@ static int stress_hdd(const stress_args_t *args)
 	flags = O_CREAT | O_RDWR | O_TRUNC | hdd_oflags;
 	fadvise_flags = hdd_flags & HDD_OPT_FADV_MASK;
 
-	if (!stress_get_setting("hdd-bytes", &hdd_bytes)) {
+	if (!stress_get_setting("hdd-bytes", &hdd_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			hdd_bytes = MAXIMIZED_FILE_SIZE;
+			hdd_bytes_total = MAXIMIZED_FILE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			hdd_bytes = MIN_HDD_BYTES;
+			hdd_bytes_total = MIN_HDD_BYTES;
+	}
+	if (hdd_bytes_total < MIN_HDD_BYTES) {
+		hdd_bytes_total = MIN_HDD_BYTES;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --hdd-bytes too small, using %" PRIu64 " instead\n",
+				args->name, hdd_bytes_total);
+	}
+	if (hdd_bytes_total > MAX_HDD_BYTES) {
+		hdd_bytes_total = MAX_HDD_BYTES;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --hdd-bytes too large, using %" PRIu64 " instead\n",
+				args->name, hdd_bytes_total);
 	}
 
-	hdd_bytes /= args->num_instances;
-	if (hdd_bytes < MIN_HDD_WRITE_SIZE)
+	hdd_bytes = hdd_bytes_total / args->instances;
+	if (hdd_bytes < MIN_HDD_WRITE_SIZE) {
 		hdd_bytes = MIN_HDD_WRITE_SIZE;
+		hdd_bytes_total = hdd_bytes * args->instances;
+	}
+	if (stress_instance_zero(args))
+		stress_fs_usage_bytes(args, hdd_bytes, hdd_bytes_total);
 
 	if (!stress_get_setting("hdd-write-size", &hdd_write_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -642,7 +746,6 @@ static int stress_hdd(const stress_args_t *args)
 			args->name, hdd_bytes);
 	}
 
-
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0)
 		return stress_exit_status((int)-ret);
@@ -658,7 +761,9 @@ static int stress_hdd(const stress_args_t *args)
 	ret = posix_memalign((void **)&alloc_buf, BUF_ALIGNMENT, (size_t)hdd_write_size);
 	if (ret || !alloc_buf) {
 		rc = stress_exit_status(errno);
-		pr_err("%s: cannot allocate buffer\n", args->name);
+		pr_err("%s: cannot allocate %zu byte buffer%s\n",
+			args->name, (size_t)hdd_write_size,
+			stress_get_memfree_str());
 		(void)stress_temp_dir_rm_args(args);
 		return rc;
 	}
@@ -667,30 +772,37 @@ static int stress_hdd(const stress_args_t *args)
 	/* Work around lack of posix_memalign */
 	alloc_buf = malloc((size_t)hdd_write_size + BUF_ALIGNMENT);
 	if (!alloc_buf) {
-		pr_err("%s: cannot allocate buffer\n", args->name);
+		pr_err("%s: cannot allocate %zu byte buffer%s\n",
+			args->name, (size_t)hdd_write_size + BUF_ALIGNMENT,
+			stress_get_memfree_str());
 		(void)stress_temp_dir_rm_args(args);
 		return rc;
 	}
 	buf = (uint8_t *)stress_align_address(alloc_buf, BUF_ALIGNMENT);
 #endif
-	(void)memset(buf, stress_mwc8(), hdd_write_size);
+	(void)shim_memset(buf, stress_mwc8(), hdd_write_size);
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		int fd;
 		struct stat statbuf;
-		uint64_t hdd_read_size;
+		uint64_t hdd_bytes_max = 0;
 		const char *fs_type;
+		size_t extents;
 
 		/*
 		 * aggressive option with no other option enables
 		 * the "work through all the options" mode
 		 */
 		if (!opts_set && (g_opt_flags & OPT_FLAGS_AGGRESSIVE)) {
-			opt_index = (opt_index + 1) % SIZEOF_ARRAY(hdd_opts);
+			opt_index++;
+			if (opt_index >= SIZEOF_ARRAY(hdd_opts))
+				opt_index = 0;
 
 			hdd_flags = hdd_opts[opt_index].flag;
 			hdd_oflags = hdd_opts[opt_index].oflag;
@@ -715,15 +827,9 @@ static int stress_hdd(const stress_args_t *args)
 			goto finish;
 		}
 
-		fs_type = stress_fs_type(filename);
+		fs_type = stress_get_fs_type(filename);
 
-#if defined(F_SET_FILE_RW_HINT) &&	\
-    defined(RWH_WRITE_LIFE_SHORT)
-		{
-			uint64_t hint = RWH_WRITE_LIFE_SHORT;
-			VOID_RET(int, fcntl(fd, F_SET_FILE_RW_HINT, &hint));
-		}
-#endif
+		stress_file_rw_hint_short(fd);
 
 		/* Exercise ftruncate or truncate */
 		if (stress_mwc1()) {
@@ -753,7 +859,7 @@ static int stress_hdd(const stress_args_t *args)
 		}
 		(void)shim_unlink(filename);
 
-		if (!keep_stressing(args)) {
+		if (UNLIKELY(!stress_continue(args))) {
 			(void)close(fd);
 			goto yielded;
 		}
@@ -769,29 +875,25 @@ static int stress_hdd(const stress_args_t *args)
 
 		/* Random Write */
 		if (hdd_flags & HDD_OPT_WR_RND) {
-			uint32_t w, z;
-
-			w = stress_mwc32();
-			z = stress_mwc32();
+			const uint32_t w = stress_mwc32();
+			const uint32_t z = stress_mwc32();
 
 			stress_mwc_set_seed(w, z);
 
 			for (i = 0; i < hdd_bytes; i += hdd_write_size) {
-				size_t j;
-				uint64_t offset = (i == 0) ?
+				const uint64_t offset = (i == 0) ?
 					hdd_bytes : stress_mwc64modn(hdd_bytes) & ~511UL;
 rnd_wr_retry:
-				if (!keep_stressing(args)) {
+				if (UNLIKELY(!stress_continue(args))) {
 					(void)close(fd);
 					goto yielded;
 				}
 
-				for (j = 0; j < hdd_write_size; j++) {
-					buf[j] = data_value(offset, j, args);
-				}
+				hdd_fill_buf(buf, hdd_write_size, offset, instance);
 
 				ret = stress_hdd_write(fd, buf, (off_t)offset,
-					hdd_write_size, hdd_flags);
+					hdd_write_size, hdd_flags,
+					&hdd_write_bytes, &hdd_write_duration);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
 						goto rnd_wr_retry;
@@ -805,25 +907,27 @@ rnd_wr_retry:
 					}
 					continue;
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
+				if (offset > hdd_bytes_max)
+					hdd_bytes_max = offset;
 			}
-
 			stress_mwc_set_seed(w, z);
 		}
 		/* Sequential Write */
 		if (hdd_flags & HDD_OPT_WR_SEQ) {
 			for (i = 0; i < hdd_bytes; i += hdd_write_size) {
-				size_t j;
 seq_wr_retry:
-				if (!keep_stressing(args)) {
+				if (UNLIKELY(!stress_continue(args))) {
 					(void)close(fd);
 					goto yielded;
 				}
 
-				for (j = 0; j < hdd_write_size; j++)
-					buf[j] = data_value(i, j, args);
+				hdd_fill_buf(buf, hdd_write_size, i, instance);
+
+				errno = 0;
 				ret = stress_hdd_write(fd, buf, (off_t)i,
-					hdd_write_size, hdd_flags);
+					hdd_write_size, hdd_flags,
+					&hdd_write_bytes, &hdd_write_duration);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
 						goto seq_wr_retry;
@@ -837,33 +941,32 @@ seq_wr_retry:
 					}
 					continue;
 				}
-				inc_counter(args);
+				/* max size is offset + bytes written */
+				hdd_bytes_max = i + ret;
+				stress_bogo_inc(args);
 			}
 		}
-
-		if (fstat(fd, &statbuf) < 0) {
+		if (shim_fstat(fd, &statbuf) < 0) {
 			pr_fail("%s: fstat failed, errno=%d (%s)%s\n",
 				args->name, errno, strerror(errno), fs_type);
 			(void)close(fd);
 			continue;
 		}
-		/* Round to write size to get no partial reads */
-		hdd_read_size = (uint64_t)statbuf.st_size -
-			((uint64_t)statbuf.st_size % hdd_write_size);
 
 		/* Sequential Read */
 		if (hdd_flags & HDD_OPT_RD_SEQ) {
 			uint64_t misreads = 0;
 			uint64_t baddata = 0;
 
-			for (i = 0; i < hdd_read_size; i += hdd_write_size) {
+			for (i = 0; i < hdd_bytes_max; i += hdd_write_size) {
 seq_rd_retry:
-				if (!keep_stressing(args)) {
+				if (UNLIKELY(!stress_continue(args))) {
 					(void)close(fd);
 					goto yielded;
 				}
 				ret = stress_hdd_read(fd, buf, (off_t)i,
-					hdd_write_size, hdd_flags);
+					hdd_write_size, hdd_flags,
+					&hdd_read_bytes, &hdd_read_duration);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
 						goto seq_rd_retry;
@@ -887,24 +990,24 @@ seq_rd_retry:
 
 						/* Write seq has written to all of the file, so it should always be OK */
 						for (j = 0; j < (size_t)ret; j++) {
-							const uint8_t v = data_value(i, j, args);
+							register const uint8_t v = data_value(i, j, instance);
 
-							if (buf[j] != v)
-								baddata++;
+							baddata += (buf[j] != v);
 						}
 					} else {
 						size_t j;
 
 						/* Write rnd has written to some of the file, so data either zero or OK */
 						for (j = 0; j < (size_t)ret; j++) {
-							const uint8_t v = data_value(i, j, args);
+							register const uint8_t v = data_value(i, j, instance);
 
-							if ((buf[j] != 0) && (buf[j] != v))
-								baddata++;
+							baddata += ((buf[j] != 0) && (buf[j] != v));
 						}
 					}
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
+				if (i > hdd_bytes_max)
+					hdd_bytes_max = i;
 			}
 			if (misreads)
 				pr_dbg("%s: %" PRIu64
@@ -919,8 +1022,8 @@ seq_rd_retry:
 			uint64_t misreads = 0;
 			uint64_t baddata = 0;
 
-			for (i = 0; i < hdd_read_size; i += hdd_write_size) {
-				size_t offset = (hdd_bytes > hdd_write_size) ?
+			for (i = 0; i < hdd_bytes_max; i += hdd_write_size) {
+				const size_t offset = (hdd_bytes > hdd_write_size) ?
 					stress_mwc64modn(hdd_bytes - hdd_write_size) & ~511UL : 0;
 
 				if (lseek(fd, (off_t)offset, SEEK_SET) < 0) {
@@ -930,12 +1033,13 @@ seq_rd_retry:
 					goto finish;
 				}
 rnd_rd_retry:
-				if (!keep_stressing(args)) {
+				if (UNLIKELY(!stress_continue(args))) {
 					(void)close(fd);
 					goto yielded;
 				}
 				ret = stress_hdd_read(fd, buf, (off_t)offset,
-					hdd_write_size, hdd_flags);
+					hdd_write_size, hdd_flags,
+					&hdd_read_bytes, &hdd_read_duration);
 				if (ret <= 0) {
 					if ((errno == EAGAIN) || (errno == EINTR))
 						goto rnd_rd_retry;
@@ -956,19 +1060,20 @@ rnd_rd_retry:
 					size_t j;
 
 					for (j = 0; j < (size_t)ret; j++) {
-						uint8_t v = data_value(offset, j, args);
+						register const uint8_t v = data_value(offset, j, instance);
+
 						if (hdd_flags & HDD_OPT_WR_SEQ) {
 							/* Write seq has written to all of the file, so it should always be OK */
 							if (buf[j] != v)
 								baddata++;
 						} else {
 							/* Write rnd has written to some of the file, so data either zero or OK */
-							if (buf[j] != 0 && buf[j] != v)
+							if ((buf[j] != 0) && (buf[j] != v))
 								baddata++;
 						}
 					}
 				}
-				inc_counter(args);
+				stress_bogo_inc(args);
 			}
 			if (misreads)
 				pr_dbg("%s: %" PRIu64
@@ -978,31 +1083,51 @@ rnd_rd_retry:
 				pr_fail("%s: incorrect data found %" PRIu64 " times\n",
 					args->name, baddata);
 		}
+		extents = stress_get_extents(fd);
+		if (extents > max_extents)
+			max_extents = extents;
 
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 yielded:
 	rc = EXIT_SUCCESS;
 finish:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
+	rate = (hdd_read_duration > 0.0) ? hdd_read_bytes / hdd_read_duration : 0.0;
+	stress_metrics_set(args, 0, "MB/sec read rate",
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
+	rate = (hdd_write_duration > 0.0) ? hdd_write_bytes / hdd_write_duration : 0.0;
+	stress_metrics_set(args, 1, "MB/sec write rate",
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
+
+	hdd_rdwr_duration = hdd_read_duration + hdd_write_duration;
+	hdd_rdwr_bytes = hdd_read_bytes + hdd_write_bytes;
+
+	rate = (hdd_rdwr_duration > 0.0) ? hdd_rdwr_bytes / hdd_rdwr_duration : 0.0;
+	stress_metrics_set(args, 2, "MB/sec read/write combined rate",
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
+
+	stress_metrics_set(args, 3, "max extents per file",
+		(double)max_extents, STRESS_METRIC_GEOMETRIC_MEAN);
+
 	free(alloc_buf);
 	(void)stress_temp_dir_rm_args(args);
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_hdd_bytes,	stress_set_hdd_bytes },
-	{ OPT_hdd_opts,		stress_set_hdd_opts },
-	{ OPT_hdd_write_size,	stress_set_hdd_write_size },
-	{ 0,			NULL },
+static const stress_opt_t opts[] = {
+	{ OPT_hdd_bytes,      "hdd-bytes",      TYPE_ID_UINT64_BYTES_FS, MIN_HDD_BYTES, MAX_HDD_BYTES, NULL },
+	{ OPT_hdd_opts,       "hdd-opts",       TYPE_ID_CALLBACK, 0, 0, stress_hdd_opts },
+	{ OPT_hdd_write_size, "hdd-write-size", TYPE_ID_UINT64_BYTES_FS, MIN_HDD_WRITE_SIZE, MAX_HDD_WRITE_SIZE, NULL },
+	END_OPT,
 };
 
-stressor_info_t stress_hdd_info = {
+const stressor_info_t stress_hdd_info = {
 	.stressor = stress_hdd,
-	.class = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_IO | CLASS_FILESYSTEM | CLASS_OS,
+	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
