@@ -138,7 +138,7 @@ again:
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		} else if (pid == 0) {
-			(void)pause();
+			(void)shim_pause();
 			_exit(0);
 		} else {
 			/* Parent */
@@ -197,7 +197,7 @@ again:
 			ret = shim_pidfd_send_signal(pidfd, 0, NULL, 0);
 			if (UNLIKELY(ret != 0)) {
 				if (errno == ENOSYS) {
-					if (args->instance == 0)
+					if (stress_instance_zero(args))
 						pr_inf_skip("%s: skipping stress test, system call is not implemented\n",
 							args->name);
 					stress_pidfd_reap(pid, pidfd);
@@ -239,7 +239,7 @@ finish:
 
 const stressor_info_t stress_pidfd_info = {
 	.stressor = stress_pidfd,
-	.class = CLASS_INTERRUPT | CLASS_OS,
+	.classifier = CLASS_INTERRUPT | CLASS_OS,
 	.supported = stress_pidfd_supported,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -254,7 +254,7 @@ static int stress_pidfd_supported(const char *name)
 
 const stressor_info_t stress_pidfd_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_INTERRUPT | CLASS_OS,
+	.classifier = CLASS_INTERRUPT | CLASS_OS,
 	.supported = stress_pidfd_supported,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
