@@ -21,15 +21,17 @@
 #include "core-mincore.h"
 #include "core-out-of-memory.h"
 
-static volatile bool do_jmp = true;
-static sigjmp_buf jmp_env;
-
 static const stress_help_t help[] = {
 	{ NULL,	"rmap N",	"start N workers that stress reverse mappings" },
 	{ NULL,	"rmap-ops N",	"stop after N rmap bogo operations" },
 	{ NULL,	NULL,		NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
+static volatile bool do_jmp = true;
+static sigjmp_buf jmp_env;
+
 typedef struct {
 	int fd;
 	double start;
@@ -136,21 +138,24 @@ static void MLOCKED_TEXT stress_rlimit_handler(int signum)
 {
 	(void)signum;
 
-	if (do_jmp)
+	if (do_jmp) {
 		siglongjmp(jmp_env, 1);
+		stress_no_return();
+	}
 }
 
 
 static int stress_rlimit_child(stress_args_t *args, void *ctxt)
 {
-	stress_rlimit_context_t *context = (stress_rlimit_context_t *)ctxt;
+	const stress_rlimit_context_t *context = (stress_rlimit_context_t *)ctxt;
 	uint8_t *stack;
 
 	stack = (uint8_t *)mmap(NULL, STRESS_MINSIGSTKSZ, PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (stack == MAP_FAILED) {
-		pr_inf("%s: cannot allocate signal stack: %d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf("%s: failed to mmap %zu byte signal stack%s, errno=%d (%s)\n",
+			args->name, (size_t)STRESS_MINSIGSTKSZ,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -205,7 +210,7 @@ static int stress_rlimit_child(stress_args_t *args, void *ctxt)
 		if (UNLIKELY((stress_time_now() - context->start) > (double)g_opt_timeout))
 			break;
 		/* Check for counter limit reached */
-		if (UNLIKELY(args->max_ops && (stress_bogo_get(args) >= args->max_ops)))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		if (LIKELY(ret == 0)) {
@@ -321,7 +326,19 @@ static int stress_rlimit(stress_args_t *args)
 
 const stressor_info_t stress_rlimit_info = {
 	.stressor = stress_rlimit,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_rlimit_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
