@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-mmap.h"
 #include "core-pthread.h"
 
 #if defined(HAVE_SEMAPHORE_H)
@@ -274,7 +275,7 @@ static int stress_sem(stress_args_t *args)
 
 	/* Wait for termination */
 	while (stress_continue(args))
-		pause();
+		(void)shim_pause();
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -309,7 +310,7 @@ static int stress_sem(stress_args_t *args)
 
 const stressor_info_t stress_sem_info = {
 	.stressor = stress_sem,
-	.class = CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
+	.classifier = CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.init = stress_sem_init,
@@ -319,7 +320,7 @@ const stressor_info_t stress_sem_info = {
 #else
 const stressor_info_t stress_sem_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
+	.classifier = CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
